﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls.DragDrop;

namespace SL.ZoomDrag
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            this.DataContext = new MainViewModel();

            RadDragAndDropManager.AddDragQueryHandler(SampleGrid, RadGridView_DragQuery);
            RadDragAndDropManager.AddDragInfoHandler(SampleGrid, RadGridView_DragInfo);

            RadDragAndDropManager.AddDropQueryHandler(MainCanvas, Canvas_DropQuery);
            RadDragAndDropManager.AddDropInfoHandler(MainCanvas, Canvas_DropInfo);
        }

        private void RadGridView_DragQuery(object sender, Telerik.Windows.Controls.DragDrop.DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
        }

        private void RadGridView_DragInfo(object sender, Telerik.Windows.Controls.DragDrop.DragDropEventArgs e)
        {
            e.Options.DragCue = new TextBlock { Text = "Dragging"};
            e.Options.Payload = e.Options.Source.DataContext;
        }

        private void Canvas_DropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
        }

        private void Canvas_DropInfo(object sender, DragDropEventArgs e)
        {
            if (e.Options.Status == DragStatus.DropComplete)
            {
                var textBlock = new TextBox { Text =((User)e.Options.Payload).UserName,IsReadOnly=true };

                Point dropPoint = GetPropPoint(e);

                MainCanvas.Children.Add(textBlock);

                Canvas.SetLeft(textBlock,dropPoint.X);
                Canvas.SetTop(textBlock, dropPoint.Y);

            }
        }

        private Point GetPropPoint(DragDropEventArgs e)
        {
            Point desiredPosition = new Point();
            Point currentDragPoint = e.Options.CurrentDragPoint;
            MatrixTransform transform = (MatrixTransform)this.MainCanvas.TransformToVisual(Application.Current.RootVisual);
            var scaleTransform = MainCanvas.RenderTransform as ScaleTransform;

            TransformGroup Transofmgroup = new TransformGroup() { Children = new TransformCollection { scaleTransform, transform } };

            Point canvasPosition = Transofmgroup.Transform(new Point());

            desiredPosition.X = (currentDragPoint.X - canvasPosition.X) / scaleTransform.ScaleY;
            desiredPosition.Y = (currentDragPoint.Y - canvasPosition.Y)/scaleTransform.ScaleY;


            return desiredPosition;
        }
    }


    public class User : INotifyPropertyChanged
    {
        private int id;

        public int Id
        {
            get
            {
                return id;
            }
            set
            {
                id = value;
                this.OnPropertyChanged("Id");
            }
        }

        private string userName;

        public string UserName
        {
            get
            {
                return userName;
            }
            set
            {
                userName = value;
                this.OnPropertyChanged("UserName");
            }
        }

        private DateTime lastLoginDate;

        public DateTime LastLoginDate
        {
            get
            {
                return lastLoginDate;
            }
            set
            {
                lastLoginDate = value;
                this.OnPropertyChanged("LastLoginDate");
            }
        }


        private bool active;

        public bool Active
        {
            get
            {
                return active;
            }
            set
            {
                active = value;
                this.OnPropertyChanged("Active");
            }
        }


        private bool approved;

        public bool Approved
        {
            get
            {
                return approved;
            }
            set
            {
                approved = value;
                this.OnPropertyChanged("Approved");
            }
        }


        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class MainViewModel
    {
        private ObservableCollection<User> data;

        public ObservableCollection<User> Data
        {
            get { return data; }
            set { data = value; }
        }

        public MainViewModel()
        {
            GenerateData(100);
        }

        private void GenerateData(int p)
        {
            this.data = new ObservableCollection<User>((
                from c in Enumerable.Range(0, p)
                select new User
                {
                    Id = c,
                    Active = c % 2 == 0,
                    Approved = c % 3 == 0,
                    LastLoginDate = DateTime.Now.AddDays(-c),
                    UserName = "User " + c
                }).ToList());
        }

    }
      
}
